@@ -1,10 +1,9 @@
-# -*- coding: utf8 -*-
+# -*- coding: utf-8 -*-
 """Medication handling code.
 
 license: GPL v2 or later
 """
 #============================================================
-__version__ = "$Revision: 1.21 $"
 __author__ = "K.Hilbert <Karsten.Hilbert@gmx.net>"
 
 import sys
@@ -16,6 +15,7 @@ import re as regex
 import subprocess
 import decimal
 from xml.etree import ElementTree as etree
+import datetime as pydt
 
 
 if __name__ == '__main__':
@@ -32,12 +32,12 @@ from Gnumed.pycommon import gmDateTime
 
 from Gnumed.business import gmATC
 from Gnumed.business import gmAllergy
+from Gnumed.business import gmCoding
 from Gnumed.business.gmDocuments import DOCUMENT_TYPE_PRESCRIPTION
 from Gnumed.business.gmDocuments import create_document_type
 
 
 _log = logging.getLogger('gm.meds')
-_log.info(__version__)
 
 #_ = lambda x:x
 DEFAULT_MEDICATION_HISTORY_EPISODE = _('Medication history')
@@ -46,7 +46,7 @@ def _on_substance_intake_modified():
 	"""Always relates to the active patient."""
 	gmHooks.run_hook_script(hook = u'after_substance_intake_modified')
 
-gmDispatcher.connect(_on_substance_intake_modified, u'substance_intake_mod_db')
+gmDispatcher.connect(_on_substance_intake_modified, u'clin.substance_intake_mod_db')
 
 #============================================================
 def drug2renal_insufficiency_url(search_term=None):
@@ -103,37 +103,7 @@ def drug2renal_insufficiency_url(search_term=None):
 	_log.debug(u'renal insufficiency URL: %s', url)
 
 	return url
-#============================================================
-# this should be in gmCoding.py
-def create_data_source(long_name=None, short_name=None, version=None, source=None, language=None):
 
-		args = {
-			'lname': long_name,
-			'sname': short_name,
-			'ver': version,
-			'src': source,
-			'lang': language
-		}
-
-		cmd = u"""select pk from ref.data_source where name_long = %(lname)s and name_short = %(sname)s and version = %(ver)s"""
-		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
-		if len(rows) > 0:
-			return rows[0]['pk']
-
-		cmd = u"""
-			INSERT INTO ref.data_source (name_long, name_short, version, source, lang)
-			VALUES (
-				%(lname)s,
-				%(sname)s,
-				%(ver)s,
-				%(src)s,
-				%(lang)s
-			)
-			returning pk
-			"""
-		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = True)
-
-		return rows[0]['pk']
 #============================================================
 # wishlist:
 # - --conf-file= for glwin.exe
@@ -349,7 +319,7 @@ class cFreeDiamsInterface(cDrugDataSourceInterface):
 		return version
 	#--------------------------------------------------------
 	def create_data_source_entry(self):
-		return create_data_source (
+		return gmCoding.create_data_source (
 			long_name = u'"FreeDiams" Drug Database Frontend',
 			short_name = u'FreeDiams',
 			version = self.get_data_source_version(),
@@ -528,7 +498,7 @@ class cFreeDiamsInterface(cDrugDataSourceInterface):
 				# do fail because __export_latest_prescription() should not have been called without patient
 				return False
 			emr = self.patient.get_emr()
-			substance_intakes = emr.get_current_substance_intake (
+			substance_intakes = emr.get_current_substance_intakes (
 				include_inactive = False,
 				include_unapproved = True
 			)
@@ -792,16 +762,16 @@ class cFreeDiamsInterface(cDrugDataSourceInterface):
 			return
 
 		for part in parts:
-			part['obj_comment'] = _('copy')
+			part['obj_comment'] = _('copy of printed prescription')
 			part.save()
 
 		xml_part = parts[-1]
 		xml_part['filename'] = u'freediams-prescription.xml'
-		xml_part['obj_comment'] = _('data')
+		xml_part['obj_comment'] = _('prescription data')
 		xml_part.save()
 
 		# are we the intended reviewer ?
-		from Gnumed.business.gmPerson import gmCurrentProvider
+		from Gnumed.business.gmStaff import gmCurrentProvider
 		me = gmCurrentProvider()
 		# if so: auto-sign the prescription
 		if xml_part['pk_intended_reviewer'] == me['pk_staff']:
@@ -1147,7 +1117,7 @@ class cGelbeListeWindowsInterface(cDrugDataSourceInterface):
 	def create_data_source_entry(self):
 		versions = self.get_data_source_version()
 
-		return create_data_source (
+		return gmCoding.create_data_source (
 			long_name = u'Medikamentendatenbank "mmi PHARMINDEX" (Gelbe Liste)',
 			short_name = u'GL/MMI',
 			version = u'Daten: %s, Preise (Onlineupdate): %s' % (versions['data'], versions['online_update']),
@@ -1563,64 +1533,131 @@ def create_consumable_substance(substance=None, atc=None, amount=None, unit=None
 def delete_consumable_substance(substance=None):
 	args = {'pk': substance}
 	cmd = u"""
-DELETE FROM ref.consumable_substance
-WHERE
-	pk = %(pk)s
-		AND
-
-	-- must not currently be used with a patient
-	NOT EXISTS (
-		SELECT 1
-		FROM clin.v_pat_substance_intake
-		WHERE pk_substance = %(pk)s
-		LIMIT 1
-	)
-		AND
-
-	-- must not currently be used with a branded drug
-	NOT EXISTS (
-		SELECT 1
-		FROM ref.lnk_substance2brand
-		WHERE fk_substance = %(pk)s
-		LIMIT 1
-	)"""
+		DELETE FROM ref.consumable_substance
+		WHERE
+			pk = %(pk)s
+				AND
+			-- must not currently be used with a patient
+			NOT EXISTS (
+				SELECT 1
+				FROM clin.v_substance_intakes		-- could be row from brand or non-brand intake, so look at both
+				WHERE pk_substance = %(pk)s
+				LIMIT 1
+			)
+				AND
+			-- must not currently be used with a branded drug
+			NOT EXISTS (
+				SELECT 1
+				FROM ref.lnk_substance2brand
+				WHERE fk_substance = %(pk)s
+				LIMIT 1
+			)
+	"""
 	gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 	return True
+
 #------------------------------------------------------------
 class cSubstanceMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 
 	_pattern = regex.compile(r'^\D+\s*\d+$', regex.UNICODE | regex.LOCALE)
-	_query1 = u"""
+
+	_normal_query = u"""
 		SELECT
-			pk::text,
-			(description || ' ' || amount || ' ' || unit) as subst
-		FROM ref.consumable_substance
-		WHERE description %(fragment_condition)s
-		ORDER BY subst
-		LIMIT 50"""
-	_query2 = u"""
+			data,
+			field_label,
+			list_label,
+			rank
+		FROM ((
+			-- first: substance intakes which match
+			SELECT
+				pk_substance AS data,
+				(description || ' ' || amount || ' ' || unit) AS field_label,
+				(description || ' ' || amount || ' ' || unit || ' (%s)') AS list_label,
+				1 AS rank
+			FROM (
+				SELECT DISTINCT ON (description, amount, unit)
+					pk_substance,
+					substance AS description,
+					amount,
+					unit
+				FROM clin.v_nonbrand_intakes
+			) AS normalized_intakes
+			WHERE description %%(fragment_condition)s
+		) UNION ALL (
+			-- consumable substances which match - but are not intakes - are second
+			SELECT
+				pk AS data,
+				(description || ' ' || amount || ' ' || unit) AS field_label,
+				(description || ' ' || amount || ' ' || unit) AS list_label,
+				2 AS rank
+			FROM ref.consumable_substance
+			WHERE
+				description %%(fragment_condition)s
+					AND
+				pk NOT IN (
+					SELECT fk_substance
+					FROM clin.substance_intake
+					WHERE fk_substance IS NOT NULL
+				)
+		)) AS candidates
+		ORDER BY rank, list_label
+		LIMIT 50""" % _('in use')
+
+	_regex_query = 	u"""
 		SELECT
-			pk::text,
-			(description || ' ' || amount || ' ' || unit) as subst
-		FROM ref.consumable_substance
-		WHERE
-			%(fragment_condition)s
-		ORDER BY subst
-		LIMIT 50"""
+			data,
+			field_label,
+			list_label,
+			rank
+		FROM ((
+			SELECT
+				pk_substance AS data,
+				(description || ' ' || amount || ' ' || unit) AS field_label,
+				(description || ' ' || amount || ' ' || unit || ' (%s)') AS list_label,
+				1 AS rank
+			FROM (
+				SELECT DISTINCT ON (description, amount, unit)
+					pk_substance,
+					substance AS description,
+					amount,
+					unit
+				FROM clin.v_nonbrand_intakes
+			) AS normalized_intakes
+			WHERE
+				%%(fragment_condition)s
+		) UNION ALL (
+			-- matching substances which are not in intakes
+			SELECT
+				pk AS data,
+				(description || ' ' || amount || ' ' || unit) AS field_label,
+				(description || ' ' || amount || ' ' || unit) AS list_label,
+				2 AS rank
+			FROM ref.consumable_substance
+			WHERE
+				%%(fragment_condition)s
+					AND
+				pk NOT IN (
+					SELECT fk_substance
+					FROM clin.substance_intake
+					WHERE fk_substance IS NOT NULL
+				)
+		)) AS candidates
+		ORDER BY rank, list_label
+		LIMIT 50""" % _('in use')
 
 	#--------------------------------------------------------
 	def getMatchesByPhrase(self, aFragment):
 		"""Return matches for aFragment at start of phrases."""
 
 		if cSubstanceMatchProvider._pattern.match(aFragment):
-			self._queries = [cSubstanceMatchProvider._query2]
+			self._queries = [cSubstanceMatchProvider._regex_query]
 			fragment_condition = """description ILIKE %(desc)s
 				AND
 			amount::text ILIKE %(amount)s"""
 			self._args['desc'] = u'%s%%' % regex.sub(r'\s*\d+$', u'', aFragment)
 			self._args['amount'] = u'%s%%' % regex.sub(r'^\D+\s*', u'', aFragment)
 		else:
-			self._queries = [cSubstanceMatchProvider._query1]
+			self._queries = [cSubstanceMatchProvider._normal_query]
 			fragment_condition = u"ILIKE %(fragment)s"
 			self._args['fragment'] = u"%s%%" % aFragment
 
@@ -1630,7 +1667,7 @@ class cSubstanceMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 		"""Return matches for aFragment at start of words inside phrases."""
 
 		if cSubstanceMatchProvider._pattern.match(aFragment):
-			self._queries = [cSubstanceMatchProvider._query2]
+			self._queries = [cSubstanceMatchProvider._regex_query]
 
 			desc = regex.sub(r'\s*\d+$', u'', aFragment)
 			desc = gmPG2.sanitize_pg_regex(expression = desc, escape_all = False)
@@ -1642,7 +1679,7 @@ class cSubstanceMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 			self._args['desc'] = u"( %s)|(^%s)" % (desc, desc)
 			self._args['amount'] = u'%s%%' % regex.sub(r'^\D+\s*', u'', aFragment)
 		else:
-			self._queries = [cSubstanceMatchProvider._query1]
+			self._queries = [cSubstanceMatchProvider._normal_query]
 			fragment_condition = u"~* %(fragment)s"
 			aFragment = gmPG2.sanitize_pg_regex(expression = aFragment, escape_all = False)
 			self._args['fragment'] = u"( %s)|(^%s)" % (aFragment, aFragment)
@@ -1653,23 +1690,24 @@ class cSubstanceMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 		"""Return matches for aFragment as a true substring."""
 
 		if cSubstanceMatchProvider._pattern.match(aFragment):
-			self._queries = [cSubstanceMatchProvider._query2]
+			self._queries = [cSubstanceMatchProvider._regex_query]
 			fragment_condition = """description ILIKE %(desc)s
 				AND
 			amount::text ILIKE %(amount)s"""
 			self._args['desc'] = u'%%%s%%' % regex.sub(r'\s*\d+$', u'', aFragment)
 			self._args['amount'] = u'%s%%' % regex.sub(r'^\D+\s*', u'', aFragment)
 		else:
-			self._queries = [cSubstanceMatchProvider._query1]
+			self._queries = [cSubstanceMatchProvider._normal_query]
 			fragment_condition = u"ILIKE %(fragment)s"
 			self._args['fragment'] = u"%%%s%%" % aFragment
 
 		return self._find_matches(fragment_condition)
+
 #============================================================
 class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 	"""Represents a substance currently taken by a patient."""
 
-	_cmd_fetch_payload = u"SELECT * FROM clin.v_pat_substance_intake WHERE pk_substance_intake = %s"
+	_cmd_fetch_payload = u"SELECT * FROM clin.v_substance_intakes WHERE pk_substance_intake = %s"
 	_cmds_store_payload = [
 		u"""UPDATE clin.substance_intake SET
 				clin_when = %(started)s,
@@ -1740,22 +1778,25 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 	#--------------------------------------------------------
 	def format_as_one_line(self, left_margin=0, date_format='%Y %b %d'):
 
-		if self._payload[self._idx['duration']] is None:
-			duration = gmTools.bool2subst (
-				self._payload[self._idx['is_long_term']],
-				_('long-term'),
-				_('short-term'),
-				_('?short-term')
-			)
+		if self._payload[self._idx['is_currently_active']]:
+			if self._payload[self._idx['duration']] is None:
+				duration = gmTools.bool2subst (
+					self._payload[self._idx['is_long_term']],
+					_('long-term'),
+					_('short-term'),
+					_('?short-term')
+				)
+			else:
+				duration = gmDateTime.format_interval (
+					self._payload[self._idx['duration']],
+					accuracy_wanted = gmDateTime.acc_days
+				)
 		else:
-			duration = gmDateTime.format_interval (
-				self._payload[self._idx['duration']],
-				accuracy_wanted = gmDateTime.acc_days
-			)
+			duration = gmDateTime.pydt_strftime(self._payload[self._idx['discontinued']], date_format)
 
 		line = u'%s%s (%s %s): %s %s%s %s (%s)' % (
 			u' ' * left_margin,
-			gmDateTime.pydt_strftime(self._payload[self._idx['started']], date_format),
+			self.medically_formatted_start,
 			gmTools.u_right_arrow,
 			duration,
 			self._payload[self._idx['substance']],
@@ -1803,8 +1844,6 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 		txt += u' ' + _('Substance: %s   [#%s]\n') % (self._payload[self._idx['substance']], self._payload[self._idx['pk_substance']])
 		txt += u' ' + _('Preparation: %s\n') % self._payload[self._idx['preparation']]
 		txt += u' ' + _('Amount per dose: %s %s') % (self._payload[self._idx['amount']], self._payload[self._idx['unit']])
-		if self.ddd is not None:
-			txt += u' (DDD: %s %s)' % (self.ddd['ddd'], self.ddd['unit'])
 		txt += u'\n'
 		txt += gmTools.coalesce(self._payload[self._idx['atc_substance']], u'', _(' ATC (substance): %s\n'))
 
@@ -1837,11 +1876,7 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 				duration = u' %s %s' % (gmTools.u_right_arrow, gmDateTime.format_interval(self._payload[self._idx['duration']], gmDateTime.acc_days))
 
 		txt += _(' Started %s%s%s\n') % (
-			gmDateTime.pydt_strftime (
-				self._payload[self._idx['started']],
-				format = date_format,
-				accuracy = gmDateTime.acc_days
-			),
+			self.medically_formatted_start,
 			duration,
 			gmTools.bool2subst(self._payload[self._idx['is_long_term']], _(' (long-term)'), _(' (short-term)'), u'')
 		)
@@ -1903,28 +1938,6 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 	#--------------------------------------------------------
 	# properties
 	#--------------------------------------------------------
-	def _get_ddd(self):
-
-		try: self.__ddd
-		except AttributeError: self.__ddd = None
-
-		if self.__ddd is not None:
-			return self.__ddd
-
-		if self._payload[self._idx['atc_substance']] is not None:
-			ddd = gmATC.atc2ddd(atc = self._payload[self._idx['atc_substance']])
-			if len(ddd) != 0:
-				self.__ddd = ddd[0]
-		else:
-			if self._payload[self._idx['atc_brand']] is not None:
-				ddd = gmATC.atc2ddd(atc = self._payload[self._idx['atc_brand']])
-				if len(ddd) != 0:
-					self.__ddd = ddd[0]
-
-		return self.__ddd
-
-	ddd = property(_get_ddd, lambda x:x)
-	#--------------------------------------------------------
 	def _get_external_code(self):
 		drug = self.containing_drug
 
@@ -1953,6 +1966,35 @@ class cSubstanceIntakeEntry(gmBusinessDBObject.cBusinessDBObject):
 
 	containing_drug = property(_get_containing_drug, lambda x:x)
 	#--------------------------------------------------------
+	def _get_medically_formatted_start(self):
+		if self._payload[self._idx['started']] is None:
+			return u''
+
+		duration_taken = gmDateTime.pydt_now_here() - self._payload[self._idx['started']]
+
+		three_months = pydt.timedelta(weeks = 13, days = 3)
+		if duration_taken < three_months:
+			return _('%s: %s ago') % (
+				gmDateTime.pydt_strftime(self._payload[self._idx['started']], '%Y %b %d', u'utf8', gmDateTime.acc_days),
+				gmDateTime.format_interval_medically(duration_taken)
+			)
+
+		five_years = pydt.timedelta(weeks = 265)
+		if duration_taken < five_years:
+			return _('%s: %s ago (%s)') % (
+				gmDateTime.pydt_strftime(self._payload[self._idx['started']], '%Y %b', u'utf8', gmDateTime.acc_months),
+				gmDateTime.format_interval_medically(duration_taken),
+				gmDateTime.pydt_strftime(self._payload[self._idx['started']], '%b %d', u'utf8', gmDateTime.acc_days)
+			)
+
+		return _('%s: %s ago (%s)') % (
+			gmDateTime.pydt_strftime(self._payload[self._idx['started']], '%Y', u'utf8', gmDateTime.acc_years),
+			gmDateTime.format_interval_medically(duration_taken),
+			gmDateTime.pydt_strftime(self._payload[self._idx['started']], '%b %d', u'utf8', gmDateTime.acc_years)
+		)
+
+	medically_formatted_start = property(_get_medically_formatted_start, lambda x:x)
+	#--------------------------------------------------------
 	def _get_parsed_schedule(self):
 		tests = [
 			# lead, trail
@@ -2049,19 +2091,22 @@ def create_substance_intake(pk_substance=None, pk_component=None, preparation=No
 def delete_substance_intake(substance=None):
 	cmd = u'delete from clin.substance_intake where pk = %(pk)s'
 	gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': {'pk': substance}}])
+
 #------------------------------------------------------------
 def format_substance_intake_notes(emr=None, output_format=u'latex', table_type=u'by-brand'):
 
-	tex  = u'\n\\noindent %s\n' % _('Additional notes')
-	tex += u'\n'
-	tex += u'\\noindent \\begin{tabularx}{\\textwidth}{|>{\\RaggedRight}X|l|>{\\RaggedRight}X|p{7.5cm}|}\n'
+	tex = u'\\noindent %s\n' % _('Additional notes')
+	tex += u'%%%% requires "\\usepackage{longtable}"\n'
+	tex += u'%%%% requires "\\usepackage{tabu}"\n'
+	tex += u'\\noindent \\begin{longtabu} to \\textwidth {|X[,L]|l|X[,L]|}\n'
 	tex += u'\\hline\n'
 	tex += u'%s {\\scriptsize (%s)} & %s & %s \\tabularnewline \n' % (_('Substance'), _('Brand'), _('Strength'), _('Aim'))
 	tex += u'\\hline\n'
+	tex += u'\\hline\n'
 	tex += u'%s\n'
-	tex += u'\\end{tabularx}\n\n'
+	tex += u'\\end{longtabu}\n'
 
-	current_meds = emr.get_current_substance_intake (
+	current_meds = emr.get_current_substance_intakes (
 		include_inactive = False,
 		include_unapproved = False,
 		order_by = u'brand, substance'
@@ -2092,17 +2137,18 @@ def format_substance_intake_notes(emr=None, output_format=u'latex', table_type=u
 #------------------------------------------------------------
 def format_substance_intake(emr=None, output_format=u'latex', table_type=u'by-brand'):
 
-	tex =  u'\\noindent %s {\\tiny (%s)\\par}\n' % (_('Medication list'), _('ordered by brand'))
-	tex += u'\n'
-	tex += u'\\noindent \\begin{tabularx}{\\textwidth}{|>{\\RaggedRight}X|>{\\RaggedRight}X|}\n'
+	tex = u'\\noindent %s {\\tiny (%s)\\par}\n' % (_('Medication list'), _('ordered by brand'))
+	tex += u'%%%% requires "\\usepackage{longtable}"\n'
+	tex += u'%%%% requires "\\usepackage{tabu}"\n'
+	tex += u'\\noindent \\begin{longtabu} to \\textwidth {|X[-1,L]|X[2.5,L]|}\n'
 	tex += u'\\hline\n'
 	tex += u'%s & %s \\tabularnewline \n' % (_('Drug'), _('Regimen / Advice'))
 	tex += u'\\hline\n'
 	tex += u'\\hline\n'
 	tex += u'%s\n'
-	tex += u'\\end{tabularx}\n'
+	tex += u'\\end{longtabu}\n'
 
-	current_meds = emr.get_current_substance_intake (
+	current_meds = emr.get_current_substance_intakes (
 		include_inactive = False,
 		include_unapproved = False,
 		order_by = u'brand, substance'
@@ -2162,6 +2208,7 @@ def format_substance_intake(emr=None, output_format=u'latex', table_type=u'by-br
 		lines.append(u'\\hline')
 
 	return tex % u'\n'.join(lines)
+
 #============================================================
 _SQL_get_drug_components = u'SELECT * FROM ref.v_drug_components WHERE %s'
 
@@ -2213,7 +2260,23 @@ def get_drug_components():
 	cmd = _SQL_get_drug_components % u'true ORDER BY brand, substance'
 	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd}], get_col_idx = True)
 	return [ cDrugComponent(row = {'data': r, 'idx': idx, 'pk_field': 'pk_component'}) for r in rows ]
+
 #------------------------------------------------------------
+_SQL_find_matching_drug_components_by_name = u"""
+SELECT
+	data,
+	field_label,
+	list_label,
+	rank
+FROM ((
+
+	) UNION ALL (
+
+	)) AS matches
+ORDER BY rank, list_label
+LIMIT 50"""
+
+
 class cDrugComponentMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 
 	_pattern = regex.compile(r'^\D+\s*\d+$', regex.UNICODE | regex.LOCALE)
@@ -2227,9 +2290,9 @@ class cDrugComponentMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 				|| r_vdc1.preparation || ' ('
 				|| r_vdc1.brand || ' ['
 					|| (
-						select array_to_string(array_agg(r_vdc2.amount), ' / ')
-						from ref.v_drug_components r_vdc2
-						where r_vdc2.pk_brand = r_vdc1.pk_brand
+						SELECT array_to_string(array_agg(r_vdc2.amount), ' / ')
+						FROM ref.v_drug_components r_vdc2
+						WHERE r_vdc2.pk_brand = r_vdc1.pk_brand
 					)
 				|| ']'
 			 || ')'
@@ -2239,9 +2302,9 @@ class cDrugComponentMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 				|| r_vdc1.preparation || ' ('
 				|| r_vdc1.brand || ' ['
 					|| (
-						select array_to_string(array_agg(r_vdc2.amount), ' / ')
-						from ref.v_drug_components r_vdc2
-						where r_vdc2.pk_brand = r_vdc1.pk_brand
+						SELECT array_to_string(array_agg(r_vdc2.amount), ' / ')
+						FROM ref.v_drug_components r_vdc2
+						WHERE r_vdc2.pk_brand = r_vdc1.pk_brand
 					)
 				|| ']'
 			 || ')'
@@ -2262,9 +2325,9 @@ class cDrugComponentMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 				|| r_vdc1.preparation || ' ('
 				|| r_vdc1.brand || ' ['
 					|| (
-						select array_to_string(array_agg(r_vdc2.amount), ' / ')
-						from ref.v_drug_components r_vdc2
-						where r_vdc2.pk_brand = r_vdc1.pk_brand
+						SELECT array_to_string(array_agg(r_vdc2.amount), ' / ')
+						FROM ref.v_drug_components r_vdc2
+						WHERE r_vdc2.pk_brand = r_vdc1.pk_brand
 					)
 				|| ']'
 			 || ')'
@@ -2274,9 +2337,9 @@ class cDrugComponentMatchProvider(gmMatchProvider.cMatchProvider_SQL2):
 				|| r_vdc1.preparation || ' ('
 				|| r_vdc1.brand || ' ['
 					|| (
-						select array_to_string(array_agg(r_vdc2.amount), ' / ')
-						from ref.v_drug_components r_vdc2
-						where r_vdc2.pk_brand = r_vdc1.pk_brand
+						SELECT array_to_string(array_agg(r_vdc2.amount), ' / ')
+						FROM ref.v_drug_components r_vdc2
+						WHERE r_vdc2.pk_brand = r_vdc1.pk_brand
 					)
 				|| ']'
 			 || ')'
@@ -2618,7 +2681,7 @@ def delete_branded_drug(brand=None):
 				AND
 			NOT EXISTS (
 				SELECT 1
-				FROM clin.v_pat_substance_intake
+				FROM clin.v_brand_intakes
 				WHERE pk_brand = %(pk)s
 				LIMIT 1
 			)
@@ -2633,7 +2696,7 @@ def delete_branded_drug(brand=None):
 				AND
 			NOT EXISTS (
 				SELECT 1
-				FROM clin.v_pat_substance_intake
+				FROM clin.v_brand_intakes
 				WHERE pk_brand = %(pk)s
 				LIMIT 1
 			)
@@ -2721,7 +2784,7 @@ if __name__ == "__main__":
 		gmPerson.set_active_patient(patient = gmPerson.cIdentity(aPK_obj = 12))
 		fd = cFreeDiamsInterface()
 		fd.patient = gmPerson.gmCurrentPatient()
-		fd.check_interactions(substances = fd.patient.get_emr().get_current_substance_intake(include_unapproved = True))
+		fd.check_interactions(substances = fd.patient.get_emr().get_current_substance_intakes(include_unapproved = True))
 	#--------------------------------------------------------
 	# generic
 	#--------------------------------------------------------
