@@ -1,4 +1,4 @@
-# -*- coding: utf8 -*-
+# -*- coding: utf-8 -*-
 """GNUmed patient objects.
 
 This is a patient object intended to let a useful client-side
@@ -50,6 +50,33 @@ __gender_idx = None
 __gender2salutation_map = None
 __gender2string_map = None
 
+
+#============================================================
+def external_id_exists(pk_issuer, value):
+	cmd = u'SELECT COUNT(1) FROM dem.lnk_identity2ext_id WHERE fk_origin = %(issuer)s AND external_id = %(val)s'
+	args = {'issuer': pk_issuer, 'val': value}
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+	return rows[0][0]
+
+#============================================================
+def person_exists(lastnames, dob, firstnames=None):
+	args = {
+		'last': lastnames,
+		'dob': dob
+	}
+	where_parts = [
+		u"lastnames = %(last)s",
+		u"dem.date_trunc_utc('day', dob) = dem.date_trunc_utc('day', %(dob)s)"
+	]
+	if firstnames is not None:
+		if firstnames.strip() != u'':
+			#where_parts.append(u"position(%(first)s in firstnames) = 1")
+			where_parts.append(u"firstnames ~* %(first)s")
+			args['first'] = u'\\m' + firstnames
+	cmd = u"""SELECT COUNT(1) FROM dem.v_basic_person WHERE %s""" % u' AND '.join(where_parts)
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+	return rows[0][0]
+
 #============================================================
 # FIXME: make this work as a mapping type, too
 class cDTO_person(object):
@@ -808,6 +835,20 @@ where id_identity = %(pat)s and id = %(pk)s"""
 				""" % (_('merged'),	gmDateTime.pydt_strftime()),
 			'args': args
 		})
+		# - same addresses
+		queries.append ({
+			'cmd': u"""
+				DELETE FROM dem.lnk_person_org_address
+				WHERE
+					id_identity = %(pat2del)s
+						AND
+					id_address IN (
+						SELECT id_address FROM dem.lnk_person_org_address d_lpoa
+						WHERE d_lpoa.id_identity = %(pat2keep)s
+					)
+			""",
+			'args': args
+		})
 
 		# generate UPDATEs
 		cmd_template = u'UPDATE %s SET %s = %%(pat2keep)s WHERE %s = %%(pat2del)s'
@@ -1015,16 +1056,9 @@ where id_identity = %(pat)s and id = %(pk)s"""
 	def link_address(self, number=None, street=None, postcode=None, urb=None, state=None, country=None, subunit=None, suburb=None, id_type=None, address=None):
 		"""Link an address with a patient, creating the address if it does not exists.
 
-		@param number The number of the address.
-		@param street The name of the street.
-		@param postcode The postal code of the address.
-		@param urb The name of town/city/etc.
-		@param state The code of the state.
-		@param country The code of the country.
 		@param id_type The primary key of the address type.
 		"""
 		if address is None:
-			# create/get address
 			address = gmDemographicRecord.create_address (
 				country = country,
 				state = state,
@@ -1040,7 +1074,7 @@ where id_identity = %(pat)s and id = %(pk)s"""
 			return None
 
 		# already linked ?
-		cmd = u"SELECT * FROM dem.lnk_person_org_address WHERE id_identity = %(pat)s AND id_address = %(adr)s"
+		cmd = u"SELECT id_address FROM dem.lnk_person_org_address WHERE id_identity = %(pat)s AND id_address = %(adr)s"
 		args = {'pat': self.pk_obj, 'adr': address['pk_address']}
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
 
@@ -1050,18 +1084,17 @@ where id_identity = %(pat)s and id = %(pk)s"""
 			cmd = u"""
 				INSERT INTO dem.lnk_person_org_address(id_identity, id_address)
 				VALUES (%(id)s, %(adr)s)
-				RETURNING *"""
+				RETURNING id_address"""
 			rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = True)
 
-		# already or now linked - needs to change type ?
-		if id_type is not None:
-			r = rows[0]
-			if r['id_type'] != id_type:
-				cmd = "UPDATE dem.lnk_person_org_address SET id_type = %(type)s WHERE id = %(id)s"
-				args = {'type': id_type, 'id': r['id']}
-				gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+		linked_adr = gmDemographicRecord.cPatientAddress(aPK_obj = rows[0]['id_address'])
 
-		return address
+		# possibly change type
+		if id_type is not None:
+			linked_adr['pk_address_type'] = id_type
+			linked_adr.save()
+
+		return linked_adr
 	#----------------------------------------------------------------------
 	def unlink_address(self, address=None, pk_address=None):
 		"""Remove an address from the patient.
@@ -1413,8 +1446,8 @@ class gmCurrentPatient(gmBorg.cBorg):
 		return None
 	#--------------------------------------------------------
 	def __register_interests(self):
-		gmDispatcher.connect(signal = u'identity_mod_db', receiver = self._on_identity_change)
-		gmDispatcher.connect(signal = u'name_mod_db', receiver = self._on_identity_change)
+		gmDispatcher.connect(signal = u'dem.identity_mod_db', receiver = self._on_identity_change)
+		gmDispatcher.connect(signal = u'dem.names_mod_db', receiver = self._on_identity_change)
 	#--------------------------------------------------------
 	def _on_identity_change(self):
 		"""Listen for patient *data* change."""
