@@ -115,7 +115,8 @@ known_schema_hashes = {
 	15: '70012ff960b77ecdff4981c94b5b55b6',
 	16: '0bcf44ca22c479b52976e5eda1de8161',
 	17: '161428ee97a00e3bf56168c3a15b7b50',
-	18: 'a0f9efcabdecfb4ddb6d8c0b69c02092'
+	18: 'a0f9efcabdecfb4ddb6d8c0b69c02092',
+	19: '419e5225259c53dd36ad80d82066ff02'
 }
 
 map_schema_hash2version = {
@@ -135,7 +136,8 @@ map_schema_hash2version = {
 	'70012ff960b77ecdff4981c94b5b55b6': 15,
 	'0bcf44ca22c479b52976e5eda1de8161': 16,
 	'161428ee97a00e3bf56168c3a15b7b50': 17,
-	'a0f9efcabdecfb4ddb6d8c0b69c02092': 18
+	'a0f9efcabdecfb4ddb6d8c0b69c02092': 18,
+	'419e5225259c53dd36ad80d82066ff02': 19
 }
 
 map_client_branch2required_db_version = {
@@ -150,7 +152,8 @@ map_client_branch2required_db_version = {
 	u'1.0': 16,		# intentional duplicate with 1.1
 	u'1.1': 16,
 	u'1.2': 17,
-	u'1.3': 18
+	u'1.3': 18,
+	u'1.4': 19
 }
 
 # get columns and data types for a given table
@@ -177,6 +180,28 @@ where
 order by
 	cols.ordinal_position"""
 
+# only works for single-column FKs but that's fine
+# needs gm-dbo, any-doc won't work
+SQL_foreign_key_name = u"""SELECT tc.constraint_schema, tc.constraint_name
+FROM
+	information_schema.table_constraints tc
+		INNER JOIN information_schema.constraint_column_usage ccu USING (constraint_catalog, constraint_schema, constraint_name)
+			INNER JOIN information_schema.key_column_usage kcu USING (constraint_catalog, constraint_schema, constraint_name)
+WHERE
+	tc.constraint_type = 'FOREIGN KEY'
+		AND
+	kcu.table_schema = %(src_schema)s
+		AND
+	kcu.table_name = %(src_tbl)s
+		AND
+	kcu.column_name = %(src_col)s
+		AND
+	ccu.table_schema = %(target_schema)s
+		AND
+	ccu.table_name = %(target_tbl)s
+		AND
+	ccu.column_name = %(target_col)s"""
+
 # =======================================================================
 # module globals API
 # =======================================================================
@@ -333,7 +358,7 @@ def __request_login_params_tui():
 	print "\nPlease enter the required login parameters:"
 	try:
 		login.host = prompted_input(prompt = "host ('' = non-TCP/IP)", default = '')
-		login.database = prompted_input(prompt = "database", default = 'gnumed_v18')
+		login.database = prompted_input(prompt = "database", default = 'gnumed_v19')
 		login.user = prompted_input(prompt = "user name", default = '')
 		tmp = 'password for "%s" (not shown): ' % login.user
 		login.password = getpass.getpass(tmp)
@@ -579,6 +604,27 @@ where
 	)
 
 	return rows
+
+#------------------------------------------------------------------------
+def get_foreign_key_names(src_schema=None, src_table=None, src_column=None, target_schema=None, target_table=None, target_column=None, link_obj=None):
+
+	args = {
+		'src_schema': src_schema,
+		'src_tbl': src_table,
+		'src_col': src_column,
+		'target_schema': target_schema,
+		'target_tbl': target_table,
+		'target_col': target_column
+	}
+
+	rows, idx = run_ro_queries (
+		link_obj = link_obj,
+		queries = [{'cmd': SQL_foreign_key_name, 'args': args}],
+		get_col_idx = False
+	)
+
+	return rows
+
 #------------------------------------------------------------------------
 def get_child_tables(schema='public', table=None, link_obj=None):
 	"""Return child tables of <table>."""
@@ -842,11 +888,11 @@ def force_user_language(language=None):
 # query runners and helpers
 # =======================================================================
 def send_maintenance_notification():
-	cmd = u'notify "db_maintenance_warning:"'
+	cmd = u'notify "db_maintenance_warning"'
 	run_rw_queries(queries = [{'cmd': cmd}], return_data = False)
 #------------------------------------------------------------------------
 def send_maintenance_shutdown():
-	cmd = u'notify "db_maintenance_disconnect:"'
+	cmd = u'notify "db_maintenance_disconnect"'
 	run_rw_queries(queries = [{'cmd': cmd}], return_data = False)
 #------------------------------------------------------------------------
 def is_pg_interval(candidate=None):
@@ -861,6 +907,67 @@ def is_pg_interval(candidate=None):
 			return True
 		except:
 			return False
+
+#------------------------------------------------------------------------
+def lock_row(link_obj=None, table=None, pk=None, exclusive=False):
+	"""Uses pg_advisory(_shared).
+
+	- locks stack upon each other and need one unlock per lock
+	- same connection:
+		- all locks succeed
+	- different connections:
+		- shared + shared succeed
+		- shared + exclusive fail
+	"""
+	_log.debug('locking row: [%s] [%s] (exclusive: %s)', table, pk, exclusive)
+	if exclusive:
+		cmd = u"""SELECT pg_try_advisory_lock('%s'::regclass::oid::int, %s)""" % (table, pk)
+	else:
+		cmd = u"""SELECT pg_try_advisory_lock_shared('%s'::regclass::oid::int, %s)""" % (table, pk)
+	rows, idx = run_ro_queries(link_obj = link_obj, queries = [{'cmd': cmd}], get_col_idx = False)
+	if rows[0][0]:
+		return True
+	_log.warning('cannot lock row: [%s] [%s] (exclusive: %s)', table, pk, exclusive)
+	return False
+
+#------------------------------------------------------------------------
+def unlock_row(link_obj=None, table=None, pk=None, exclusive=False):
+	"""Uses pg_advisory_unlock(_shared).
+
+	- each lock needs one unlock
+	"""
+	_log.debug('trying to unlock row: [%s] [%s] (exclusive: %s)', table, pk, exclusive)
+	if exclusive:
+		cmd = u"SELECT pg_advisory_unlock('%s'::regclass::oid::int, %s)" % (table, pk)
+	else:
+		cmd = u"SELECT pg_advisory_unlock_shared('%s'::regclass::oid::int, %s)" % (table, pk)
+	rows, idx = run_ro_queries(link_obj = link_obj, queries = [{'cmd': cmd}], get_col_idx = False)
+	if rows[0][0]:
+		return True
+	_log.warning('cannot unlock row: [%s] [%s] (exclusive: %s)', table, pk, exclusive)
+	return False
+
+#------------------------------------------------------------------------
+def row_is_locked(table=None, pk=None):
+	"""Looks at pk_locks
+
+	- does not take into account locks other than 'advisory', however
+	"""
+	cmd = u"""SELECT EXISTS (
+		SELECT 1 FROM pg_locks WHERE
+			classid = '%s'::regclass::oid::int
+				AND
+			objid = %s
+				AND
+			locktype = 'advisory'
+	)""" % (table, pk)
+	rows, idx = run_ro_queries(queries = [{'cmd': cmd}], get_col_idx = False)
+	if rows[0][0]:
+		_log.debug('row is locked: [%s] [%s]', table, pk)
+		return True
+	_log.debug('row is NOT locked: [%s] [%s]', table, pk)
+	return False
+
 #------------------------------------------------------------------------
 def bytea2file(data_query=None, filename=None, chunk_size=0, data_size=None, data_size_query=None):
 	outfile = file(filename, 'wb')
@@ -921,13 +1028,14 @@ def bytea2file_object(data_query=None, file_obj=None, chunk_size=0, data_size=No
 	needed_chunks, remainder = divmod(data_size, chunk_size)
 	_log.debug('# of chunks: %s; remainder: %s bytes', needed_chunks, remainder)
 
-	# try setting "bytea_output"
-	# - fails if not necessary
-	# - succeeds if necessary
-	try:
-		run_ro_queries(link_obj = conn, queries = [{'cmd': u"set bytea_output to 'escape'"}])
-	except dbapi.ProgrammingError:
-		_log.debug('failed to set bytea_output to "escape", not necessary')
+#	# since we now require PG 9.1 we can disable this workaround:
+#	# try setting "bytea_output"
+#	# - fails if not necessary
+#	# - succeeds if necessary
+#	try:
+#		run_ro_queries(link_obj = conn, queries = [{'cmd': u"set bytea_output to 'escape'"}])
+#	except dbapi.ProgrammingError:
+#		_log.debug('failed to set bytea_output to "escape", not necessary')
 
 	# retrieve chunks, skipped if data size < chunk size,
 	# does this not carry the danger of cutting up multi-byte escape sequences ?
@@ -965,14 +1073,19 @@ def bytea2file_object(data_query=None, file_obj=None, chunk_size=0, data_size=No
 	conn.rollback()
 	return True
 #------------------------------------------------------------------------
-def file2bytea(query=None, filename=None, args=None, conn=None):
+def file2bytea(query=None, filename=None, args=None, conn=None, file_md5=None):
 	"""Store data from a file into a bytea field.
 
 	The query must:
 	- be in unicode
 	- contain a format spec identifying the row (eg a primary key)
 	  matching <args> if it is an UPDATE
-	- contain a format spec %(data)s::bytea
+	- contain a format spec " <field> = %(data)s::bytea"
+
+	The query CAN return the MD5 of the inserted data:
+		RETURNING md5(<field>) AS md5
+	in which case it will compare it to the md5
+	of the file.
 	"""
 	# read data from file
 	infile = file(filename, "rb")
@@ -990,12 +1103,25 @@ def file2bytea(query=None, filename=None, args=None, conn=None):
 	else:
 		close_conn = False
 
-	run_rw_queries(link_obj = conn, queries = [{'cmd': query, 'args': args}], end_tx = True)
+	rows, idx = run_rw_queries(link_obj = conn, queries = [{'cmd': query, 'args': args}], end_tx = False, return_data = (file_md5 is not None))
+
+	success_status = True
+	if file_md5 is None:
+		conn.commit()
+	else:
+		db_md5 = rows[0]['md5']
+		if file_md5 != db_md5:
+			conn.rollback()
+			success_status = False
+			_log.error('MD5 sums of data file and database BYTEA field do not match: [file::%s] <> [DB::%s]', file_md5, db_md5)
+		else:
+			conn.commit()
+			_log.debug('MD5 sums of data file and database BYTEA field match: [file::%s] = [DB::%s]', file_md5, db_md5)
 
 	if close_conn:
 		conn.close()
 
-	return
+	return success_status
 #------------------------------------------------------------------------
 def sanitize_pg_regex(expression=None, escape_all=False):
 	"""Escape input for use in a PostgreSQL regular expression.
@@ -1480,6 +1606,7 @@ def get_connection(dsn=None, readonly=True, encoding=None, verbose=False, pooled
 
 	# - transaction isolation level
 	if readonly:
+		# alter-database default, checked at connect, no need to set now
 		iso_level = u'read committed'
 	else:
 		conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_SERIALIZABLE)
@@ -1494,22 +1621,22 @@ def get_connection(dsn=None, readonly=True, encoding=None, verbose=False, pooled
 
 	conn.commit()
 
-	# FIXME: remove this whole affair once either 9.0 is standard (Ubuntu 10 LTS is
-	# FIXME: PG 8.4, however!) or else when psycopg2 supports a workaround
-	#
-	# - bytea data format
-	# PG 9.0 switched to - by default - using "hex" rather than "escape",
-	# however, psycopg2's linked with a pre-9.0 libpq do assume "escape"
-	# as the transmission mode for bytea output,
-	# so try to set this setting back to "escape",
-	# if that's not possible the reason will be that PG < 9.0 does not support
-	# that setting - which also means we don't need it and can ignore the
-	# failure
-	cmd = "set bytea_output to 'escape'"
-	try:
-		curs.execute(cmd)
-	except dbapi.ProgrammingError:
-		_log.error('cannot set bytea_output format')
+#	# FIXME: remove this whole affair once either 9.0 is standard (Ubuntu 10 LTS is
+#	# FIXME: PG 8.4, however!) or else when psycopg2 supports a workaround
+#	#
+#	# - bytea data format
+#	# PG 9.0 switched to - by default - using "hex" rather than "escape",
+#	# however, psycopg2's linked with a pre-9.0 libpq do assume "escape"
+#	# as the transmission mode for bytea output,
+#	# so try to set this setting back to "escape",
+#	# if that's not possible the reason will be that PG < 9.0 does not support
+#	# that setting - which also means we don't need it and can ignore the
+#	# failure
+#	cmd = "set bytea_output to 'escape'"
+#	try:
+#		curs.execute(cmd)
+#	except dbapi.ProgrammingError:
+#		_log.error('cannot set bytea_output format')
 
 	curs.close()
 	conn.commit()
@@ -1615,9 +1742,10 @@ def sanity_check_database_settings():
 		u'full_page_writes': [u'on', u'data loss/corruption', False],
 		u'lc_messages': [u'C', u'suboptimal error detection', False],
 		u'password_encryption': [u'on', u'breach of confidentiality', False],
-		u'regex_flavor': [u'advanced', u'query breakage', False],					# 9.0 doesn't support this anymore, default now advanced anyway
+		#u'regex_flavor': [u'advanced', u'query breakage', False],					# 9.0 doesn't support this anymore, default now advanced anyway
 		u'synchronous_commit': [u'on', u'data loss/corruption', False],
-		u'sql_inheritance': [u'on', u'query breakage, data loss/corruption', True]
+		u'sql_inheritance': [u'on', u'query breakage, data loss/corruption', True],
+		u'ignore_checksum_failure': [u'off', u'data loss/corruption', False]		# starting with PG 9.3
 	}
 
 	from Gnumed.pycommon import gmCfg2
@@ -1773,13 +1901,6 @@ psycopg2.extensions.register_type(psycopg2._psycopg.UNICODEARRAY)
 # check in 0.9:
 psycopg2.extensions.register_adapter(pydt.datetime, cAdapterPyDateTime)
 
-## remove for 0.9
-#try:
-#	import mx.DateTime as mxDT
-##	psycopg2.extensions.register_adapter(mxDT.DateTimeType, cAdapterMxDateTime)
-#except ImportError:
-#	_log.warning('cannot import mx.DateTime')
-
 # do NOT adapt *lists* to "... IN (*) ..." syntax because we want
 # them adapted to "... ARRAY[]..." so we can support PG arrays
 
@@ -2098,11 +2219,70 @@ if __name__ == "__main__":
 		cmd = u"SELECT %(dt)s"
 
 		#cmd = u"SELECT 'infinity'::timestamp with time zone"
+
+		cmd = u"""
+SELECT to_timestamp (foofoo,'YYMMDD.HH24MI') FROM (
+	SELECT REGEXP_REPLACE (
+		't1.130729.0902.tif',			-- string
+		E'(.1)\.([0-9\.]+)(\.tif)',		-- pattern
+		E'\\\\2'						-- replacement
+	) AS foofoo
+) AS foo"""
 		rows, idx = run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
 		print rows
+		print rows[0]
+		print rows[0][0]
 	#--------------------------------------------------------------------
 	def test_schema_exists():
 		print schema_exists()
+	#--------------------------------------------------------------------
+	def test_row_locks():
+		row_is_locked(table = 'dem.identity', pk = 12)
+
+		print "1st connection:"
+		print " locked:", row_is_locked(table = 'dem.identity', pk = 12)
+		print " 1st shared lock succeeded:", lock_row(table = 'dem.identity', pk = 12, exclusive = False)
+		print " locked:", row_is_locked(table = 'dem.identity', pk = 12)
+
+		print "   2nd shared lock should succeed:", lock_row(table = 'dem.identity', pk = 12, exclusive = False)
+		print "   `-> unlock succeeded:", unlock_row(table = 'dem.identity', pk = 12, exclusive = False)
+		print " locked:", row_is_locked(table = 'dem.identity', pk = 12)
+		print "   exclusive lock should succeed:", lock_row(table = 'dem.identity', pk = 12, exclusive = True)
+		print "   `-> unlock succeeded:", unlock_row(table = 'dem.identity', pk = 12, exclusive = True)
+		print " locked:", row_is_locked(table = 'dem.identity', pk = 12)
+
+		print "2nd connection:"
+		conn = get_raw_connection(readonly=True)
+		print " shared lock should succeed:", lock_row(link_obj = conn, table = 'dem.identity', pk = 12, exclusive = False)
+		print " `-> unlock succeeded:", unlock_row(link_obj = conn, table = 'dem.identity', pk = 12, exclusive = False)
+		print " locked:", row_is_locked(table = 'dem.identity', pk = 12)
+		print " exclusive lock succeeded ?", lock_row(link_obj = conn, table = 'dem.identity', pk = 12, exclusive = True), "(should fail)"
+		print " locked:", row_is_locked(table = 'dem.identity', pk = 12)
+
+		print "1st connection:"
+		print " unlock succeeded:", unlock_row(table = 'dem.identity', pk = 12, exclusive = False)
+		print " locked:", row_is_locked(table = 'dem.identity', pk = 12)
+
+		print "2nd connection:"
+		print " exclusive lock should succeed", lock_row(link_obj = conn, table = 'dem.identity', pk = 12, exclusive = True)
+		print " locked:", row_is_locked(table = 'dem.identity', pk = 12)
+		print "  shared lock should succeed:", lock_row(link_obj = conn, table = 'dem.identity', pk = 12, exclusive = False)
+		print "  `-> unlock succeeded:", unlock_row(link_obj = conn, table = 'dem.identity', pk = 12, exclusive = False)
+		print " locked:", row_is_locked(table = 'dem.identity', pk = 12)
+		print " unlock succeeded:", unlock_row(link_obj = conn, table = 'dem.identity', pk = 12, exclusive = False)
+		print " locked:", row_is_locked(table = 'dem.identity', pk = 12)
+
+		conn.close()
+	#--------------------------------------------------------------------
+	def test_get_foreign_key_names():
+		print get_foreign_key_names (
+			src_schema = 'dem',
+			src_table = 'names',
+			src_column = 'id_identity',
+			target_schema = 'dem',
+			target_table = 'identity',
+			target_column = 'pk'
+		)
 
 	#--------------------------------------------------------------------
 	# run tests
@@ -2121,6 +2301,8 @@ if __name__ == "__main__":
 	#test_set_user_language()
 	#test_get_schema_revision_history()
 	#test_run_query()
-	test_schema_exists()
+	#test_schema_exists()
+	#test_get_foreign_key_names()
+	test_row_locks()
 
 # ======================================================================
