@@ -1,7 +1,6 @@
 """GNUmed medical document handling widgets.
 """
 #================================================================
-__version__ = "$Revision: 1.187 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
 import os.path
@@ -12,6 +11,7 @@ import logging
 
 
 import wx
+import wx.lib.mixins.treemixin as treemixin
 
 
 if __name__ == '__main__':
@@ -31,18 +31,18 @@ from Gnumed.business import gmPerson
 from Gnumed.business import gmStaff
 from Gnumed.business import gmDocuments
 from Gnumed.business import gmEMRStructItems
-from Gnumed.business import gmSurgery
+from Gnumed.business import gmPraxis
 
 from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxpython import gmRegetMixin
 from Gnumed.wxpython import gmPhraseWheel
 from Gnumed.wxpython import gmPlugin
 from Gnumed.wxpython import gmEMRStructWidgets
+from Gnumed.wxpython import gmEncounterWidgets
 from Gnumed.wxpython import gmListWidgets
 
 
 _log = logging.getLogger('gm.ui')
-_log.info(__version__)
 
 
 default_chunksize = 1 * 1024 * 1024		# 1 MB
@@ -196,6 +196,7 @@ def save_files_as_new_document(parent=None, filenames=None, document_type=None,
 #----------------------
 gmDispatcher.connect(signal = u'import_document_from_file', receiver = _save_file_as_new_document)
 gmDispatcher.connect(signal = u'import_document_from_files', receiver = _save_files_as_new_document)
+
 #============================================================
 class cDocumentCommentPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	"""Let user select a document comment from all existing comments."""
@@ -253,6 +254,7 @@ LIMIT 25"""],
 		self.picklist_delay = 50
 
 		self.SetToolTipString(_('Enter a comment on the document.'))
+
 #============================================================
 # document type widgets
 #============================================================
@@ -263,6 +265,7 @@ def manage_document_types(parent=None):
 
 	dlg = cEditDocumentTypesDlg(parent = parent)
 	dlg.ShowModal()
+
 #============================================================
 from Gnumed.wxGladeWidgets import wxgEditDocumentTypesDlg
 
@@ -289,7 +292,7 @@ class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 		self._LCTRL_doc_type.set_column_widths()
 	#--------------------------------------------------------
 	def __register_interests(self):
-		gmDispatcher.connect(signal = u'doc_type_mod_db', receiver = self._on_doc_type_mod_db)
+		gmDispatcher.connect(signal = u'blobs.doc_type_mod_db', receiver = self._on_doc_type_mod_db)
 	#--------------------------------------------------------
 	def _on_doc_type_mod_db(self):
 		wx.CallAfter(self.repopulate_ui)
@@ -415,6 +418,7 @@ class cEditDocumentTypesPnl(wxgEditDocumentTypesPnl.wxgEditDocumentTypesPnl):
 		wx.EndBusyCursor()
 
 		return
+
 #============================================================
 class cDocumentTypeSelectionPhraseWheel(gmPhraseWheel.cPhraseWheel):
 	"""Let user select a document type."""
@@ -476,6 +480,7 @@ ORDER BY q1.rank, q1.list_label"""]
 				value = doc_type,
 				data = pk
 			)
+
 #============================================================
 # document review widgets
 #============================================================
@@ -737,6 +742,7 @@ class cReviewDocPartDlg(wxgReviewDocPartDlg.wxgReviewDocPartDlg):
 		else:
 			self._PRW_doc_comment.set_context(context = 'pk_doc_type', val = pk_doc_type)
 		return True
+
 #============================================================
 def acquire_images_from_capture_device(device=None, calling_window=None):
 
@@ -881,7 +887,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 			dbcfg = gmCfg.cCfgSQL()
 			allow_empty = bool(dbcfg.get2 (
 				option =  u'horstspace.scan_index.allow_partless_documents',
-				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 				bias = 'user',
 				default = False
 			))
@@ -937,7 +943,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 			dbcfg = gmCfg.cCfgSQL()
 			device = dbcfg.get2 (
 				option =  'external.xsane.default_device',
-				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 				bias = 'workplace',
 				default = ''
 			)
@@ -1149,7 +1155,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		generate_uuid = bool (
 			cfg.get2 (
 				option = 'horstspace.scan_index.generate_doc_uuid',
-				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 				bias = 'user',
 				default = False
 			)
@@ -1209,7 +1215,7 @@ class cScanIdxDocsPnl(wxgScanIdxPnl.wxgScanIdxPnl, gmPlugin.cPatientChange_Plugi
 		show_id = bool (
 			cfg.get2 (
 				option = 'horstspace.scan_index.show_doc_id',
-				workplace = gmSurgery.gmCurrentPractice().active_workplace,
+				workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 				bias = 'user'
 			)
 		)
@@ -1274,7 +1280,7 @@ def display_document_part(parent=None, part=None):
 	chunksize = int(
 	cfg.get2 (
 		option = "horstspace.blob_export_chunk_size",
-		workplace = gmSurgery.gmCurrentPractice().active_workplace,
+		workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 		bias = 'workplace',
 		default = 2048
 	))
@@ -1282,7 +1288,7 @@ def display_document_part(parent=None, part=None):
 	# shall we force blocking during view ?
 	block_during_view = bool( cfg.get2 (
 		option = 'horstspace.document_viewer.block_during_view',
-		workplace = gmSurgery.gmCurrentPractice().active_workplace,
+		workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 		bias = 'user',
 		default = None
 	))
@@ -1309,7 +1315,7 @@ def display_document_part(parent=None, part=None):
 	# 4: if no review by responsible reviewer
 	review_after_display = int(cfg.get2 (
 		option = 'horstspace.document_viewer.review_after_display',
-		workplace = gmSurgery.gmCurrentPractice().active_workplace,
+		workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 		bias = 'user',
 		default = 3
 	))
@@ -1328,6 +1334,7 @@ def display_document_part(parent=None, part=None):
 			review_document_part(parent = parent, part = part)
 
 	return True
+
 #============================================================
 def manage_documents(parent=None, msg=None, single_selection=True):
 
@@ -1406,9 +1413,9 @@ class cSelectablySortedDocTreePnl(wxgSelectablySortedDocTreePnl.wxgSelectablySor
 		self._doc_tree.sort_mode = 'type'
 		self._doc_tree.SetFocus()
 		self._rbtn_sort_by_type.SetValue(True)
+
 #============================================================
-class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
-	# FIXME: handle expansion state
+class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin, treemixin.ExpansionState):
 	"""This wx.TreeCtrl derivative displays a tree view of stored medical documents.
 
 	It listens to document and patient changes and updates itself accordingly.
@@ -1439,6 +1446,8 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		self.root = None
 		self.__sort_mode = 'age'
 
+		self.__expanded_nodes = None
+
 		self.__build_context_menus()
 		self.__register_interests()
 		self._schedule_data_reget()
@@ -1460,7 +1469,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 	#--------------------------------------------------------
 	def _get_sort_mode(self):
 		return self.__sort_mode
-	#-----
+
 	def _set_sort_mode(self, mode):
 		if mode is None:
 			mode = 'age'
@@ -1472,13 +1481,14 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 			raise ValueError('invalid document tree sort mode [%s], valid modes: %s' % (mode, cDocTree._sort_modes))
 
 		self.__sort_mode = mode
+		self.__expanded_nodes = None
 
 		curr_pat = gmPerson.gmCurrentPatient()
 		if not curr_pat.connected:
 			return
 
 		self._schedule_data_reget()
-	#-----
+
 	sort_mode = property(_get_sort_mode, _set_sort_mode)
 	#--------------------------------------------------------
 	# reget-on-paint API
@@ -1506,8 +1516,8 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 
 		gmDispatcher.connect(signal = u'pre_patient_selection', receiver = self._on_pre_patient_selection)
 		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
-		gmDispatcher.connect(signal = u'doc_mod_db', receiver = self._on_doc_mod_db)
-		gmDispatcher.connect(signal = u'doc_page_mod_db', receiver = self._on_doc_page_mod_db)
+		gmDispatcher.connect(signal = u'blobs.doc_med_mod_db', receiver = self._on_doc_mod_db)
+		gmDispatcher.connect(signal = u'blobs.doc_obj_mod_db', receiver = self._on_doc_page_mod_db)
 	#--------------------------------------------------------
 	def __build_context_menus(self):
 
@@ -1542,6 +1552,10 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		self.__part_context_menu.Append(ID, _('Mail part'))
 		wx.EVT_MENU(self.__part_context_menu, ID, self.__mail_part)
 
+		ID = wx.NewId()
+		self.__part_context_menu.Append(ID, _('Export part'))
+		wx.EVT_MENU(self.__part_context_menu, ID, self.__export_part_to_disk)
+
 		self.__part_context_menu.AppendSeparator()			# so we can append some items
 
 		# --- doc context menu ---
@@ -1551,6 +1565,10 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		self.__doc_context_menu.Append(ID, _('%s Sign/Edit properties') % u'\u270D')
 		wx.EVT_MENU(self.__doc_context_menu, ID, self.__review_curr_part)
 
+		ID = wx.NewId()
+		self.__doc_context_menu.Append(ID, _('Delete document'))
+		wx.EVT_MENU(self.__doc_context_menu, ID, self.__delete_document)
+
 		self.__doc_context_menu.AppendSeparator()
 
 		item = self.__doc_context_menu.Append(-1, _('Add parts'))
@@ -1574,10 +1592,6 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 
 		self.__doc_context_menu.AppendSeparator()
 
-		ID = wx.NewId()
-		self.__doc_context_menu.Append(ID, _('Delete document'))
-		wx.EVT_MENU(self.__doc_context_menu, ID, self.__delete_document)
-
 		ID = wx.NewId()
 		self.__doc_context_menu.Append(ID, _('Access external original'))
 		wx.EVT_MENU(self.__doc_context_menu, ID, self.__access_external_original)
@@ -1762,12 +1776,18 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		self.__sort_nodes()
 		self.SelectItem(self.root)
 
-		# FIXME: apply expansion state if available or else ...
-		# FIXME: ... uncollapse to default state
+		# restore expansion state
+		if self.__expanded_nodes is not None:
+			self.ExpansionState = self.__expanded_nodes
+		# but always expand root node
 		self.Expand(self.root)
-		if self.__sort_mode in ['episode', 'type', 'issue']:
-			for key in intermediate_nodes.keys():
-				self.Expand(intermediate_nodes[key])
+		# if no expansion state available then
+		# expand intermediate nodes as well
+		if self.__expanded_nodes is None:
+			# but only if there are any
+			if self.__sort_mode in ['episode', 'type', 'issue']:
+				for key in intermediate_nodes.keys():
+					self.Expand(intermediate_nodes[key])
 
 		wx.EndBusyCursor()
 
@@ -1897,16 +1917,14 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 	# event handlers
 	#------------------------------------------------------------------------
 	def _on_doc_mod_db(self, *args, **kwargs):
-		# FIXME: remember current expansion state
+		self.__expanded_nodes = self.ExpansionState
 		wx.CallAfter(self._schedule_data_reget)
 	#------------------------------------------------------------------------
 	def _on_doc_page_mod_db(self, *args, **kwargs):
-		# FIXME: remember current expansion state
+		self.__expanded_nodes = self.ExpansionState
 		wx.CallAfter(self._schedule_data_reget)
 	#------------------------------------------------------------------------
 	def _on_pre_patient_selection(self, *args, **kwargs):
-		# FIXME: self.__store_expansion_history_in_db
-
 		# empty out tree
 		if self.root is not None:
 			self.DeleteAllItems()
@@ -1914,6 +1932,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 	#------------------------------------------------------------------------
 	def _on_post_patient_selection(self, *args, **kwargs):
 		# FIXME: self.__load_expansion_history_from_db (but not apply it !)
+		self.__expanded_nodes = None
 		self._schedule_data_reget()
 	#------------------------------------------------------------------------
 	def _on_activate(self, event):
@@ -2051,7 +2070,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		chunksize = int(
 		cfg.get2 (
 			option = "horstspace.blob_export_chunk_size",
-			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 			bias = 'workplace',
 			default = default_chunksize
 		))
@@ -2059,7 +2078,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		# shall we force blocking during view ?
 		block_during_view = bool( cfg.get2 (
 			option = 'horstspace.document_viewer.block_during_view',
-			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 			bias = 'user',
 			default = None
 		))
@@ -2087,7 +2106,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		# 4: if no review by responsible reviewer
 		review_after_display = int(cfg.get2 (
 			option = 'horstspace.document_viewer.review_after_display',
-			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 			bias = 'user',
 			default = 3
 		))
@@ -2123,8 +2142,11 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		)
 		if target_doc is None:
 			return
-		self.__curr_node_data['pk_doc'] = target_doc['pk_doc']
-		self.__curr_node_data.save()
+		if not self.__curr_node_data.reattach(pk_doc = target_doc['pk_doc']):
+			gmGuiHelpers.gm_show_error (
+				aMessage = _('Cannot move document part.'),
+				aTitle = _('Moving document part')
+			)
 	#--------------------------------------------------------
 	def __delete_part(self, evt):
 		delete_it = gmGuiHelpers.gm_show_question (
@@ -2188,7 +2210,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		# determine database export chunk size
 		chunksize = int(cfg.get2 (
 			option = "horstspace.blob_export_chunk_size",
-			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 			bias = 'workplace',
 			default = default_chunksize
 		))
@@ -2220,6 +2242,15 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 				) % {'action': action, 'l10n_action': l10n_action},
 				_('Processing document part: %s') % l10n_action
 			)
+		else:
+			if action == 'mail':
+				curr_pat = gmPerson.gmCurrentPatient()
+				emr = curr_pat.emr
+				emr.add_clin_narrative (
+					soap_cat = None,
+					note =  _('document part handed over to email program: %s') % self.__curr_node_data.format(single_line = True),
+					episode = self.__curr_node_data['pk_episode']
+				)
 	#--------------------------------------------------------
 	def __print_part(self, evt):
 		self.__process_part(action = u'print', l10n_action = _('print'))
@@ -2230,10 +2261,57 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 	def __mail_part(self, evt):
 		self.__process_part(action = u'mail', l10n_action = _('mail'))
 	#--------------------------------------------------------
+	def __export_part_to_disk(self, evt):
+		"""Export document part into directory."""
+
+		dlg = wx.DirDialog (
+			parent = self,
+			message = _('Save document part to directory ...'),
+			defaultPath = os.path.expanduser(os.path.join('~', 'gnumed')),
+			style = wx.DD_DEFAULT_STYLE
+		)
+		result = dlg.ShowModal()
+		dirname = dlg.GetPath()
+		dlg.Destroy()
+
+		if result != wx.ID_OK:
+			return True
+
+		wx.BeginBusyCursor()
+
+		pat = gmPerson.gmCurrentPatient()
+		fname = self.__curr_node_data.get_useful_filename (
+			patient = pat,
+			make_unique = True,
+			directory = dirname
+		)
+
+		cfg = gmCfg.cCfgSQL()
+
+		# determine database export chunk size
+		chunksize = int(cfg.get2 (
+			option = "horstspace.blob_export_chunk_size",
+			workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
+			bias = 'workplace',
+			default = default_chunksize
+		))
+
+		fname = self.__curr_node_data.export_to_file (
+			aChunkSize = chunksize,
+			filename = fname,
+			target_mime = None
+		)
+
+		wx.EndBusyCursor()
+
+		gmDispatcher.send(signal = 'statustext', msg = _('Successfully exported document part as [%s].') % fname)
+
+		return True
+	#--------------------------------------------------------
 	# document level context menu handlers
 	#--------------------------------------------------------
 	def __select_encounter(self, evt):
-		enc = gmEMRStructWidgets.select_encounters (
+		enc = gmEncounterWidgets.select_encounters (
 			parent = self,
 			patient = gmPerson.gmCurrentPatient()
 		)
@@ -2244,7 +2322,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 	#--------------------------------------------------------
 	def __edit_encounter_details(self, evt):
 		enc = gmEMRStructItems.cEncounter(aPK_obj = self.__curr_node_data['pk_encounter'])
-		gmEMRStructWidgets.edit_encounter(parent = self, encounter = enc)
+		gmEncounterWidgets.edit_encounter(parent = self, encounter = enc)
 	#--------------------------------------------------------
 	def __process_doc(self, action=None, l10n_action=None):
 
@@ -2275,7 +2353,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		# determine database export chunk size
 		chunksize = int(cfg.get2 (
 			option = "horstspace.blob_export_chunk_size",
-			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 			bias = 'workplace',
 			default = default_chunksize
 		))
@@ -2390,12 +2468,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		- into subdirectory named after patient
 		"""
 		pat = gmPerson.gmCurrentPatient()
-		dname = '%s-%s%s' % (
-			self.__curr_node_data['l10n_type'],
-			self.__curr_node_data['clin_when'].strftime('%Y-%m-%d'),
-			gmTools.coalesce(self.__curr_node_data['ext_ref'], '', '-%s').replace(' ', '_')
-		)
-		def_dir = os.path.expanduser(os.path.join('~', 'gnumed', 'export', 'docs', pat['dirname'], dname))
+		def_dir = os.path.expanduser(os.path.join('~', 'gnumed', pat['dirname']))
 		gmTools.mkdir(def_dir)
 
 		dlg = wx.DirDialog (
@@ -2418,7 +2491,7 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		# determine database export chunk size
 		chunksize = int(cfg.get2 (
 			option = "horstspace.blob_export_chunk_size",
-			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			workplace = gmPraxis.gmCurrentPraxisBranch().active_workplace,
 			bias = 'workplace',
 			default = default_chunksize
 		))
@@ -2432,11 +2505,11 @@ class cDocTree(wx.TreeCtrl, gmRegetMixin.cRegetOnPaintMixin):
 		return True
 	#--------------------------------------------------------
 	def __delete_document(self, evt):
-		result = gmGuiHelpers.gm_show_question (
+		delete_it = gmGuiHelpers.gm_show_question (
 			aMessage = _('Are you sure you want to delete the document ?'),
 			aTitle = _('Deleting document')
 		)
-		if result is True:
+		if delete_it is True:
 			curr_pat = gmPerson.gmCurrentPatient()
 			emr = curr_pat.get_emr()
 			enc = emr.active_encounter
