@@ -23,6 +23,7 @@ import types
 import logging
 import thread
 import time
+import re as regex
 
 
 import wx
@@ -60,16 +61,20 @@ def get_choices_from_list (
 	- refresh_callback: (listctrl)
 	- list_tooltip_callback: (item data)
 
-	- left/middle/right_extra_button: (label, tooltip, <callback>)
-		<callback> is called with item_data as the only argument
+	- left/middle/right_extra_button: (label, tooltip, <callback> [, wants_list_ctrl])
+		wants_list_ctrl is optional
+		<callback> is called with item_data (or listctrl) as the only argument
 
 	returns:
 		on [CANCEL]: None
 		on [OK]:
 			if any items selected:
-				list of selected items
+				if single_selection:
+					the data of the selected item
+				else:
+					list of data of selected items
 			else:
-				if can_return_empty is True:
+				if can_return_empty is True AND [OK] button was pressed:
 					empty list
 				else:
 					None
@@ -250,7 +255,10 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		self._LCTRL_items.SetFocus()
 	#------------------------------------------------------------
 	def _on_left_extra_button_pressed(self, event):
-		item_data = self._LCTRL_items.get_selected_item_data(only_one=True)
+		if self.__left_extra_button_wants_list:
+			item_data = self._LCTRL_items
+		else:
+			item_data = self._LCTRL_items.get_selected_item_data(only_one=True)
 		if not self.__left_extra_button_callback(item_data):
 			self._LCTRL_items.SetFocus()
 			return
@@ -266,7 +274,10 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		self._LCTRL_items.SetFocus()
 	#------------------------------------------------------------
 	def _on_middle_extra_button_pressed(self, event):
-		item_data = self._LCTRL_items.get_selected_item_data(only_one=True)
+		if self.__middle_extra_button_wants_list:
+			item_data = self._LCTRL_items
+		else:
+			item_data = self._LCTRL_items.get_selected_item_data(only_one=True)
 		if not self.__middle_extra_button_callback(item_data):
 			self._LCTRL_items.SetFocus()
 			return
@@ -282,7 +293,10 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		self._LCTRL_items.SetFocus()
 	#------------------------------------------------------------
 	def _on_right_extra_button_pressed(self, event):
-		item_data = self._LCTRL_items.get_selected_item_data(only_one=True)
+		if self.__right_extra_button_wants_list:
+			item_data = self._LCTRL_items
+		else:
+			item_data = self._LCTRL_items.get_selected_item_data(only_one=True)
 		if not self.__right_extra_button_callback(item_data):
 			self._LCTRL_items.SetFocus()
 			return
@@ -320,12 +334,19 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 			self._BTN_extra_left.Enable(False)
 			self._BTN_extra_left.Hide()
 			self.__left_extra_button_callback = None
+			self.__left_extra_button_wants_list = False
 			return
 
-		(label, tooltip, callback) = definition
+		if len(definition) == 3:
+			(label, tooltip, callback) = definition
+			wants_list = False
+		else:
+			(label, tooltip, callback, wants_list) = definition
+
 		if not callable(callback):
 			raise ValueError('<left extra button> callback is not a callable: %s' % callback)
 		self.__left_extra_button_callback = callback
+		self.__left_extra_button_wants_list = wants_list
 		self._BTN_extra_left.SetLabel(label)
 		self._BTN_extra_left.SetToolTipString(tooltip)
 		self._BTN_extra_left.Enable(True)
@@ -338,12 +359,19 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 			self._BTN_extra_middle.Enable(False)
 			self._BTN_extra_middle.Hide()
 			self.__middle_extra_button_callback = None
+			self.__middle_extra_button_wants_list = False
 			return
 
-		(label, tooltip, callback) = definition
+		if len(definition) == 3:
+			(label, tooltip, callback) = definition
+			wants_list = False
+		else:
+			(label, tooltip, callback, wants_list) = definition
+
 		if not callable(callback):
 			raise ValueError('<middle extra button> callback is not a callable: %s' % callback)
 		self.__middle_extra_button_callback = callback
+		self.__middle_extra_button_wants_list = wants_list
 		self._BTN_extra_middle.SetLabel(label)
 		self._BTN_extra_middle.SetToolTipString(tooltip)
 		self._BTN_extra_middle.Enable(True)
@@ -356,12 +384,19 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 			self._BTN_extra_right.Enable(False)
 			self._BTN_extra_right.Hide()
 			self.__right_extra_button_callback = None
+			self.__right_extra_button_wants_list = False
 			return
 
-		(label, tooltip, callback) = definition
+		if len(definition) == 3:
+			(label, tooltip, callback) = definition
+			wants_list = False
+		else:
+			(label, tooltip, callback, wants_list) = definition
+
 		if not callable(callback):
 			raise ValueError('<right extra button> callback is not a callable: %s' % callback)
 		self.__right_extra_button_callback = callback
+		self.__right_extra_button_wants_list = wants_list
 		self._BTN_extra_right.SetLabel(label)
 		self._BTN_extra_right.SetToolTipString(tooltip)
 		self._BTN_extra_right.Enable(True)
@@ -741,8 +776,9 @@ class cItemPickerDlg(wxgItemPickerDlg.wxgItemPickerDlg):
 		else:
 			self._LBL_msg.SetLabel(msg)
 
+		self.allow_duplicate_picks = True
+
 		self._LCTRL_left.activate_callback = self.__pick_selected
-		#self._LCTRL_left.item_tooltip_callback = self.__on_get_item_tooltip
 		self.__extra_button_callback = None
 
 		self._LCTRL_left.SetFocus()
@@ -816,19 +852,32 @@ class cItemPickerDlg(wxgItemPickerDlg.wxgItemPickerDlg):
 
 		right_items = self._LCTRL_right.get_string_items()
 		right_data = self._LCTRL_right.get_item_data()
-
-		right_items.extend(self._LCTRL_left.get_selected_string_items(only_one = False))
-		self._LCTRL_right.set_string_items(items = right_items)
-		del right_items
-
 		if right_data is None:
-			self._LCTRL_right.set_data(data = self._LCTRL_left.get_selected_item_data(only_one = False))
-		else:
-			right_data.extend(self._LCTRL_left.get_selected_item_data(only_one = False))
-			self._LCTRL_right.set_data(data = right_data)
-		del right_data
+			right_data = []
 
+		selected_items = self._LCTRL_left.get_selected_string_items(only_one = False)
+		selected_data = self._LCTRL_left.get_selected_item_data(only_one = False)
+
+		if self.allow_duplicate_picks:
+			right_items.extend(selected_items)
+			right_data.extend(selected_data)
+			self._LCTRL_right.set_string_items(items = right_items)
+			self._LCTRL_right.set_data(data = right_data)
+			self._LCTRL_right.set_column_widths()
+#			print u'%s <-> %s (items)' % (self._LCTRL_left.ItemCount, self._LCTRL_right.ItemCount)
+#			print u'%s <-> %s (data)' % (len(self._LCTRL_left.data), len(self._LCTRL_right.data))
+			return
+
+		for sel_item, sel_data in zip(selected_items, selected_data):
+			if sel_item in right_items:
+				continue
+			right_items.append(sel_item)
+			right_data.append(sel_data)
+		self._LCTRL_right.set_string_items(items = right_items)
+		self._LCTRL_right.set_data(data = right_data)
 		self._LCTRL_right.set_column_widths()
+#		print u'%s <-> %s (items)' % (self._LCTRL_left.ItemCount, self._LCTRL_right.ItemCount)
+#		print u'%s <-> %s (data)' % (len(self._LCTRL_left.data), len(self._LCTRL_right.data))
 	#------------------------------------------------------------
 	def __remove_selected_picks(self):
 		if self._LCTRL_right.get_selected_items(only_one = True) == -1:
@@ -839,6 +888,9 @@ class cItemPickerDlg(wxgItemPickerDlg.wxgItemPickerDlg):
 
 		if self._LCTRL_right.GetItemCount() == 0:
 			self._BTN_right2left.Enable(False)
+
+#		print u'%s <-> %s (items)' % (self._LCTRL_left.ItemCount, self._LCTRL_right.ItemCount)
+#		print u'%s <-> %s (data)' % (len(self._LCTRL_left.data), len(self._LCTRL_right.data))
 	#------------------------------------------------------------
 	# event handlers
 	#------------------------------------------------------------
@@ -864,24 +916,33 @@ class cItemPickerDlg(wxgItemPickerDlg.wxgItemPickerDlg):
 	#------------------------------------------------------------
 	def _on_extra_button_pressed(self, event):
 		self.__extra_button_callback()
-#		item_data = self._LCTRL_items.get_selected_item_data(only_one=True)
-#		if not self.__left_extra_button_callback(item_data):
-#			self._LCTRL_items.SetFocus()
-#			return
-#		if self.refresh_callback is None:
-#			self._LCTRL_items.SetFocus()
-#			return
-#		wx.BeginBusyCursor()
-#		try:
-#			self.refresh_callback(lctrl = self._LCTRL_items)
-#		finally:
-#			wx.EndBusyCursor()
-#		self._LCTRL_items.set_column_widths()
-#		self._LCTRL_items.SetFocus()
-#================================================================
-class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
+	#------------------------------------------------------------
+	def _set_left_item_tooltip_callback(self, callback):
+		self._LCTRL_left.item_tooltip_callback = callback
 
-	# FIXME: searching by typing
+	left_item_tooltip_callback = property(lambda x:x, _set_left_item_tooltip_callback)
+	#------------------------------------------------------------
+	def _set_right_item_tooltip_callback(self, callback):
+		self._LCTRL_right.item_tooltip_callback = callback
+
+	right_item_tooltip_callback = property(lambda x:x, _set_right_item_tooltip_callback)
+
+#================================================================
+class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin, listmixins.ColumnSorterMixin):
+
+	# sorting: at set_string_items() time all items will be
+	# adorned with their initial row number as wxPython data,
+	# this is used later for a) sorting and b) to access
+	# GNUmed data objects associated with rows,
+	# the latter are ordered in initial row number order
+	# at set_data() time
+
+	map_item_idx2data_idx = wx.ListCtrl.GetItemData
+
+	sort_order_tags = {
+		True: u' [\u03b1\u0391 \u2192 \u03c9\u03A9]',
+		False: u' [\u03c9\u03A9 \u2192 \u03b1\u0391]'
+	}
 
 	def __init__(self, *args, **kwargs):
 
@@ -897,17 +958,34 @@ class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 		wx.ListCtrl.__init__(self, *args, **kwargs)
 		listmixins.ListCtrlAutoWidthMixin.__init__(self)
 
+		# required for column sorting
+		self._invalidate_sorting_metadata()					# must be called after each (external/direct) list item update
+		listmixins.ColumnSorterMixin.__init__(self, 0)		# must be called again after adding columns (why ?)
+		self.__secondary_sort_col = None
+		# for debugging sorting:
+		#self.Bind(wx.EVT_LIST_COL_CLICK, self._on_col_click, self)
+
 		self.__widths = None
 		self.__data = None
 		self.__activate_callback = None
 		self.__rightclick_callback = None
 
-		self.Bind(wx.EVT_MOTION, self._on_mouse_motion)
 		self.__item_tooltip_callback = None
 		self.__tt_last_item = None
 		self.__tt_static_part = _("""Select the items you want to work on.
 
 A discontinuous selection may depend on your holding down a platform-dependent modifier key (<ctrl>, <alt>, etc) or key combination (eg. <ctrl-shift> or <ctrl-alt>) while clicking.""")
+		self.Bind(wx.EVT_MOTION, self._on_mouse_motion)
+
+		self.__next_line_to_search = 0
+		self.__search_data = None
+		self.__search_dlg = None
+		self.__searchable_cols = None
+#		self.Bind(wx.EVT_KILL_FOCUS, self._on_lost_focus)
+		self.Bind(wx.EVT_CHAR, self._on_char)
+		self.Bind(wx.EVT_FIND_CLOSE, self._on_search_dlg_closed)
+		self.Bind(wx.EVT_FIND, self._on_search_first_match)
+		self.Bind(wx.EVT_FIND_NEXT, self._on_search_next_match)
 	#------------------------------------------------------------
 	# setters
 	#------------------------------------------------------------
@@ -922,6 +1000,8 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 			return
 		for idx in range(len(columns)):
 			self.InsertColumn(idx, columns[idx])
+
+		self._invalidate_sorting_metadata()
 	#------------------------------------------------------------
 	def set_column_widths(self, widths=None):
 		"""Set the column width policy.
@@ -958,7 +1038,9 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 		"""All item members must be unicode()able or None."""
 
 		wx.BeginBusyCursor()
+		self._invalidate_sorting_metadata()
 
+		# remove existing items
 		loop = 0
 		while True:
 			if loop > 3:
@@ -984,6 +1066,7 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 			wx.EndBusyCursor()
 			return
 
+		# insert new items
 		for item in items:
 			try:
 				item[0]
@@ -1008,22 +1091,29 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 				col_val = unicode(item)
 				row_num = self.InsertStringItem(index = sys.maxint, label = col_val)
 
+		# set data to be a copy of items
 		self.data = items
 
 		wx.EndBusyCursor()
 	#------------------------------------------------------------
 	def set_data(self, data=None):
-		"""<data must be a list corresponding to the item indices>"""
+		"""<data> assumed to be a list corresponding to the item indices"""
 		if data is not None:
 			item_count = self.GetItemCount()
 			if len(data) != item_count:
 				_log.debug('<data> length (%s) must be equal to number of list items (%s)  (%s, thread [%s])', len(data), item_count, self.debug, thread.get_ident())
+			for item_idx in range(len(data)):
+				self.SetItemData(item_idx, item_idx)
 		self.__data = data
 		self.__tt_last_item = None
+		# string data (rows/visible list items) not modified,
+		# so no need to call _update_sorting_metadata
 		return
 
 	def _get_data(self):
-		return self.__data
+		# slower than "return self.__data" but helps with detecting
+		# problems with len(__data)<>self.GetItemCount()
+		return self.get_item_data() 		# returns all data if item_idx is None
 
 	data = property(_get_data, set_data)
 	#------------------------------------------------------------
@@ -1033,7 +1123,6 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 			return
 		for idx in selections:
 			self.Select(idx = idx, on = 1)
-			#self.SetItemState(idx, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
 
 	def __get_selections(self):
 		if self.__is_single_selection:
@@ -1092,14 +1181,17 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 
 		return items
 	#------------------------------------------------------------
-	def get_item_data(self, item_idx = None):
+	def get_item_data(self, item_idx=None):
 		if self.__data is None:	# this isn't entirely clean
 			return None
 
 		if item_idx is not None:
-			return self.__data[item_idx]
+			return self.__data[self.map_item_idx2data_idx(item_idx)]
 
-		return [ self.__data[item_idx] for item_idx in range(self.GetItemCount()) ]
+		# if <idx> is None return all data up to item_count,
+		# in case of len(__data) <> self.GetItemCount() this
+		# gives the chance to figure out what is going on
+		return [ self.__data[self.map_item_idx2data_idx(item_idx)] for item_idx in range(self.GetItemCount()) ]
 	#------------------------------------------------------------
 	def get_selected_item_data(self, only_one=False):
 
@@ -1109,14 +1201,14 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 			idx = self.GetFirstSelected()
 			if idx == -1:
 				return None
-			return self.__data[idx]
+			return self.__data[self.map_item_idx2data_idx(idx)]
 
 		data = []
 		if self.__data is None:
 			return data
 		idx = self.GetFirstSelected()
 		while idx != -1:
-			data.append(self.__data[idx])
+			data.append(self.__data[self.map_item_idx2data_idx(idx)])
 			idx = self.GetNextSelected(idx)
 
 		return data
@@ -1125,10 +1217,15 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 		self.Select(idx = self.GetFirstSelected(), on = 0)
 	#------------------------------------------------------------
 	def remove_item(self, item_idx=None):
+		# do NOT remove the corresponding data because even if
+		# the item pointing to this data instance is gone all
+		# other items will still point to their corresponding
+		# *initial* row numbers
+		#if self.__data is not None:
+		#	del self.__data[self.map_item_idx2data_idx(item_idx)]
 		self.DeleteItem(item_idx)
-		if self.__data is not None:
-			del self.__data[item_idx]
 		self.__tt_last_item = None
+		self._invalidate_sorting_metadata()
 	#------------------------------------------------------------
 	# event handlers
 	#------------------------------------------------------------
@@ -1142,6 +1239,36 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 		if self.__rightclick_callback is not None:
 			self.__rightclick_callback(event)
 	#------------------------------------------------------------
+	def _on_char(self, evt):
+
+		if evt.GetModifiers() != wx.MOD_CMD:
+			evt.Skip()
+			return
+
+		if unichr(evt.GetRawKeyCode()) != u'f':
+			evt.Skip()
+			return
+
+		if self.__search_dlg is not None:
+			self.__search_dlg.Close()
+			return
+
+		if self.__searchable_cols is None:
+			self.searchable_columns = None
+
+		if len(self.__searchable_cols) == 0:
+			return
+
+		if self.__search_data is None:
+			self.__search_data = wx.FindReplaceData()
+		self.__search_dlg = wx.FindReplaceDialog (
+			self,
+			self.__search_data,
+			_('Search in list'),
+			wx.FR_NOUPDOWN | wx.FR_NOMATCHCASE | wx.FR_NOWHOLEWORD
+		)
+		self.__search_dlg.Show(True)
+	#------------------------------------------------------------
 	def _on_mouse_motion(self, event):
 		"""Update tooltip on mouse motion.
 
@@ -1217,7 +1344,7 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 
 		dyna_tt = None
 		if self.__item_tooltip_callback is not None:
-			dyna_tt = self.__item_tooltip_callback(self.__data[item_idx])
+			dyna_tt = self.__item_tooltip_callback(self.__data[self.map_item_idx2data_idx(item_idx)])
 
 		if dyna_tt is None:
 			self.SetToolTipString(self.__tt_static_part)
@@ -1225,6 +1352,45 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 
 		self.SetToolTipString(dyna_tt)
 	#------------------------------------------------------------
+	# search related methods
+	#------------------------------------------------------------
+	def _on_search_dlg_closed(self, evt):
+		self.__search_dlg.Destroy()
+		self.__search_dlg = None
+	#------------------------------------------------------------
+#	def _on_lost_focus(self, evt):
+#		evt.Skip()
+#		if self.__search_dlg is None:
+#			return
+##		print self.FindFocus()
+##		print self.__search_dlg
+#		#self.__search_dlg.Close()
+	#------------------------------------------------------------
+	def __on_search_match(self, search_term):
+		for row_idx in range(self.__next_line_to_search, self.ItemCount):
+			for col_idx in range(self.ColumnCount):
+				if col_idx not in self.__searchable_cols:
+					continue
+				col_val = self.GetItem(row_idx, col_idx).GetText()
+				if regex.search(search_term, col_val, regex.U | regex.I) is not None:
+					self.Select(row_idx)
+					self.EnsureVisible(row_idx)
+					if row_idx == self.ItemCount - 1:
+						# wrap around
+						self.__next_line_to_search = 0
+					else:
+						self.__next_line_to_search = row_idx + 1
+					return True
+		# wrap around
+		self.__next_line_to_search = 0
+		return False
+	#------------------------------------------------------------
+	def _on_search_first_match(self, evt):
+		self.__on_search_match(evt.GetFindString())
+	#------------------------------------------------------------
+	def _on_search_next_match(self, evt):
+		self.__on_search_match(evt.GetFindString())
+	#------------------------------------------------------------
 	# properties
 	#------------------------------------------------------------
 	def _get_activate_callback(self):
@@ -1266,6 +1432,110 @@ A discontinuous selection may depend on your holding down a platform-dependent m
 	# the callback must return None if no item tooltip is to be shown
 	# otherwise it must return a string (possibly with \n)
 	item_tooltip_callback = property(lambda x:x, _set_item_tooltip_callback)
+	#------------------------------------------------------------
+	def _set_searchable_cols(self, cols):
+		# zero-based list of which columns to search
+		if cols is None:
+			self.__searchable_cols = range(self.ColumnCount)
+			return
+		# weed out columns to be searched which
+		# don't exist and uniquify them
+		new_cols = {}
+		for col in cols:
+			if col < self.ColumnCount:
+				new_cols[col] = True
+		self.__searchable_cols = new_cols.keys()
+
+	searchable_columns = property(lambda x:x, _set_searchable_cols)
+	#------------------------------------------------------------
+	# ColumnSorterMixin API
+	#------------------------------------------------------------
+	def GetListCtrl(self):
+		if self.itemDataMap is None:
+			self._update_sorting_metadata()
+		return self								# required
+	#------------------------------------------------------------
+	def OnSortOrderChanged(self):
+		self._cleanup_column_headers()
+		# annotate sort column
+		col_idx, is_ascending = self.GetSortState()
+		col_state = self.GetColumn(col_idx)
+		col_state.m_text += self.sort_order_tags[is_ascending]
+		self.SetColumn(col_idx, col_state)
+	#------------------------------------------------------------
+	def GetSecondarySortValues(self, primary_sort_col, item1_idx, item2_idx):
+		if self.__secondary_sort_col is None:
+			return (item1_idx, item2_idx)
+		if self.__secondary_sort_col == primary_sort_col:
+			return (item1_idx, item2_idx)
+		val1 = self.itemDataMap[item1_idx][self.__secondary_sort_col]
+		val2 = self.itemDataMap[item2_idx][self.__secondary_sort_col]
+		order = cmp(val1, val2)
+		if order == 0:
+			return (item1_idx, item2_idx)
+		# make the secondary column always sort ascending
+		currently_ascending = self._colSortFlag[primary_sort_col]
+		if currently_ascending:
+			val1, val2 = min(val1, val2), max(val1, val2)
+		else:
+			val1, val2 = max(val1, val2), min(val1, val2)
+		return (val1, val2)
+	#------------------------------------------------------------
+	def _generate_map_for_sorting(self):
+		dict2sort = {}
+		item_count = self.GetItemCount()
+		if item_count == 0:
+			return dict2sort
+		col_count = self.GetColumnCount()
+		for item_idx in range(item_count):
+			dict2sort[item_idx] = ()
+			if col_count == 0:
+				continue
+			for col_idx in range(col_count):
+				dict2sort[item_idx] += (self.GetItem(item_idx, col_idx).GetText(), )
+
+		return dict2sort
+	#------------------------------------------------------------
+	def _cleanup_column_headers(self):
+		for col_idx in range(self.ColumnCount):
+			col_state = self.GetColumn(col_idx)
+			if col_state.m_text.endswith(self.sort_order_tags[True]):
+				col_state.m_text = col_state.m_text[:-len(self.sort_order_tags[True])]
+			if col_state.m_text.endswith(self.sort_order_tags[False]):
+				col_state.m_text = col_state.m_text[:-len(self.sort_order_tags[False])]
+			self.SetColumn(col_idx, col_state)
+	#------------------------------------------------------------
+	def _invalidate_sorting_metadata(self):
+		self.itemDataMap = None
+		self.SetColumnCount(self.GetColumnCount())
+		self._cleanup_column_headers()
+	#------------------------------------------------------------
+	def _update_sorting_metadata(self):
+		# MUST have this name
+		self.itemDataMap = self._generate_map_for_sorting()
+	#------------------------------------------------------------
+	def _on_col_click(self, event):
+		# for debugging:
+		# print "column clicked : %s" % (event.GetColumn())
+		# column, order = self.GetSortState()
+		# print "column %s sort %s" % (column, order)
+		# print self._colSortFlag
+		# print self.itemDataMap
+		event.Skip()
+	#------------------------------------------------------------
+	def __get_secondary_sort_col(self):
+		return self.__secondary_sort_col
+
+	def __set_secondary_sort_col(self, col):
+		if col is None:
+			self.__secondary_sort_col = None
+			return
+		if col > self.GetColumnCount():
+			raise ValueError('cannot secondary-sort on col [%s], there are only [%s] columns', col, self.GetColumnCount())
+		self.__secondary_sort_col = col
+
+	secondary_sort_column = property(__get_secondary_sort_col, __set_secondary_sort_col)
+
 #================================================================
 # main
 #----------------------------------------------------------------
